// -*- Mode: c++ -*-
// $Id: cvec4t.h,v 1.4 2000/07/20 15:44:04 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/include/cvec4t.h,v $

/* Subdivide V2.0
   Copyright (C) 2000 Henning Biermann, Denis Zorin, NYU

This file is part of Subdivide.

Subdivide is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Subdivide is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Subdivide; see the file COPYING.  If not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#ifndef	__CVEC4T_H__
#define	__CVEC4T_H__

#include <assert.h>
#include <math.h>
#include "compat.h"

template <class F> class CVec4T;

// another VC 6.0 problem: would not accept this definition 
// after the class

template <class F>
istream& operator>> ( istream& is, CVec4T<F>& v) {
  is >> v.x();
  is >> v.y();
  is >> v.z();
  is >> v.w();
  return is;
}

template <class F>
ostream& operator<<( ostream& os, const CVec4T<F>& v) {
  return os << v(0) << " " << v(1) << " " << v(2) << " " << v(3);
}


//: template for vectors of length 4
template <class F>
class CVec4T {
public:
  enum{ X = 0, Y = 1, Z = 2, W = 3 };
  CVec4T( void ){ v[X] = 0; v[Y] = 0; v[Z] = 0; v[W] = 0; }
  CVec4T( const CVec4T& c )
  { v[X] = c.v[X],v[Y] = c.v[Y],v[Z] = c.v[Z],v[W] = c.v[W]; }
  CVec4T( const F& a, const F& b, const F& c, const F& d )
  { v[X] = a, v[Y] = b; v[Z] = c; v[W] = d; }
  /*
  CVec4T( const CVec3T<F>& c )
  { v[X] = c.x(),v[Y] = c.y(),v[Z] = c.z(),v[W] = 0; }
  */
  //: the argument is required to be zero; assumes that 
  // conversion of 0 to the type F works
  CVec4T( int j)
  { assert(j == 0); v[X] = F(0); v[Y] = F(0); v[Z] = F(0); v[W] = F(0); }

  CVec4T( const F* a )
    { v[X] = a[X], v[Y] = a[Y], v[Z] = a[Z], v[W] = a[W]; }
  operator F*( void )
    { return &v[X]; }
  operator const F*( void ) const
    { return &v[X]; }

  bool read(istream& is) { return is >> *this; }
  /*
  CVec4T& operator=( const CVec3T<F>& c )
  { v[X] = c.x();v[Y] = c.y();v[Z] = c.z(); return *this; }
  */
  CVec4T& operator=( const CVec4T& c )
  { v[X] = c.v[X];v[Y] = c.v[Y];v[Z] = c.v[Z];v[W] = c.v[W]; return *this; }
  CVec4T operator+( const CVec4T& c ) const
  { return CVec4T( v[X] + c.v[X], v[Y] + c.v[Y], v[Z] + c.v[Z], v[W]+c.v[W]); }
  CVec4T operator-( const CVec4T& c ) const
  { return CVec4T( v[X] - c.v[X], v[Y] - c.v[Y], v[Z] - c.v[Z], v[W]-c.v[W]); }
  CVec4T operator*( const F& s ) const
  { return CVec4T( s * v[X], s * v[Y], s*v[Z], s*v[W] ); }
  CVec4T operator*( const CVec4T& c ) const
  { return CVec4T(v[X] * c.v[X], v[Y] * c.v[Y], v[Z] * c.v[Z], v[W]*c.v[W]); }
  friend CVec4T operator*( const F& s, const CVec4T& c )
  { return CVec4T( s * c.v[X], s * c.v[Y], s * c.v[Z], s*c.v[W] ); }
  CVec4T operator/( const F& s ) const
    { return CVec4T( v[X] / s, v[Y] / s , v[Z] / s, v[W] / s ); }  
  CVec4T& operator+=( const CVec4T& c )
  { v[X] += c.v[X], v[Y] += c.v[Y]; v[Z] += c.v[Z]; v[W] += c.v[W];
    return *this; }
  CVec4T& operator-=( const CVec4T& c )
  { v[X] -= c.v[X], v[Y] -= c.v[Y]; v[Z] -= c.v[Z]; v[W] -= c.v[W];
    return *this; }
  CVec4T& operator*=( const F& s )
    { v[X] *= s, v[Y] *= s,v[Z] *= s, v[W] *= s; return *this; }
  CVec4T& operator/=( const F& s )
    { v[X] /= s, v[Y] /= s, v[Z] /= s, v[W] /= s; return *this; }
  CVec4T operator-( void ) const
    { return CVec4T( -v[X], -v[Y], -v[Z], -v[W] ); }

  void plus_aX(F a, const CVec4T& u) 
    { v[X] += a*u[X]; v[Y] += a*u[X]; v[Z] += a*u[Z]; v[W] += a*u[W]; }

  F& x( void ) { return v[X]; }
  const F& x( void ) const { return v[X]; }
  F& y( void ) { return v[Y]; }
  const F& y( void ) const { return v[Y]; }
  F& z( void ) { return v[Z]; }
  const F& z( void ) const { return v[Z]; }
  F& w( void ) { return v[W]; }
  const F& w( void ) const { return v[W]; }
  F& operator() (const int i)      { return v[i]; }
  const F& operator() (const int i) const { return v[i]; }
  //: returns v + t(v1 - v), where v is *this 
  CVec4T lerp( const CVec4T& v1, const F& t ) const
    { return CVec4T( v[X] + t * ( v1.v[X] - v[X] ),
		     v[Y] + t * ( v1.v[Y] - v[Y] ), 
		     v[Z] + t * ( v1.v[Z] - v[Z] ),
		     v[W] + t * ( v1.v[W] - v[W] )); }
  //: returns the componentwise minimum
  CVec4T min( const CVec4T& o ) const
    { F a = ::min( v[X], o.v[X] );
      F b = ::min( v[Y], o.v[Y] );
      F c = ::min( v[Z], o.v[Z] );
      F d = ::min( v[W], o.v[W] );
      return CVec4T( a, b, c, d ); }
  //: returns the componentwise maximum
  CVec4T max( const CVec4T& o ) const
    { F a = ::max( v[X], o.v[X] );
      F b = ::max( v[Y], o.v[Y] );
      F c = ::max( v[Z], o.v[Z] );
      F d = ::max( v[W], o.v[W] );
      return CVec4T( a, b, c, d); }

  /*
  F dot( const CVec3T<F>& c ) const
    { return v[X] * c.x() + v[Y] * c.y()+ v[Z] * c.z(); }
  */
  F dot( const CVec4T& c ) const
    { return v[X] * c.v[X] + v[Y] * c.v[Y]+ v[Z] * c.v[Z]+ v[W] * c.v[W]; }
  F dot( void ) const
    { return v[X] * v[X] + v[Y] * v[Y] + v[Z] * v[Z] + v[W] * v[W]; }
  //: l1 norm (sum of abs values of the components)
  F l1( void ) const
    { F a = fabs( v[X] ); a += fabs( v[Y] );a += fabs( v[Z] ); 
      a += fabs( v[W] ); return a; }
  //: l infinity norm (max of abs values of the components)
  F linfty( void ) const
    { F a = fabs( v[X] ); a = ::max( a, F(fabs( v[Y] )) );
    a = ::max( a, F(fabs( v[Z] )) ); a = ::max( a, F(fabs( v[W] )) );
      return  a; }
  //: l infinity norm (sqrt of the sum of squares )
  F l2( void ) const { return sqrt( dot() ); }
  //: for nonzero vectors, returns the normalized vector
  CVec4T dir( void ) const {
    F a = l2();
    if( ( a  == 0) ) return *this;
    else return *this / a;
  }

  int normalize( void )
    { F mag = l2(); return (mag == 0) ? 0 : ( *this *= 1 / mag, 1 ); }
  //:  length |*this -c|
  F dist( const CVec4T& c ) const { return ( *this - c ).l2(); }

  CVec4T abs() {
    return CVec4T( fabs(v[X]),  fabs(v[Y]), fabs(v[Z]), fabs(v[W]) );
  }

  //: conversion of different vectors 
  // note that it is a template    memeber function (parameterzed by G) of
  // the template  class parameterized by F; not all compilers can handle this
  // so it is removed for now (gcc 2.8.1 does not understand it)
  //  template <class G> CVec4T( const CVec4T<G>& u ) { v[0] = u.x(); v[1] = u.y();
//  v[2] = u.z();  }

 protected:
 private:
  F v[4];
};

typedef CVec4T<float> cvec4f;
typedef CVec4T<double> cvec4d;

#endif	/* __CVEC4T_H__ */

