// -*- Mode: c++ -*-
// $Id: viewer.cpp,v 1.5 2000/07/31 19:42:54 jianbo Exp $
// $Source: /tools/cvs-repos/illustrate/src/viewer.cpp,v $

#if defined(_WIN32) || defined(__sgi)
#include <float.h>
#endif
#include <stdio.h>
#include <string.h>
#include <GL/glut.h>
#include "viewer.h"

const float Viewer::depthOffset = 0.0001f;

vector<Viewer*> Viewer::_viewer;

// create window, set callbacks
Viewer::Viewer(char *title, int width, int height)
  : _width(width), _height(height) {
  if (title) strcpy(_title, title);
  else sprintf(_title, " Viewer %d", _viewer.size());

  glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutInitWindowSize(_width, _height);
  _winId = glutCreateWindow(_title);
  glutDisplayFunc(_display);
  glutReshapeFunc(_reshape);
  glutMouseFunc(_mouse);
  glutMotionFunc(_motion);
  glutKeyboardFunc(_keyboard);
  glEnable(GL_DEPTH_TEST);
  glViewport(0, 0, _width, _height);

  _viewer.push_back(this);
}

// delete window
Viewer::~Viewer() {
  for (vector<Viewer*>::iterator it = _viewer.begin();
       it != _viewer.end(); it++) {
    if ((*it)->_winId == _winId) {
      _viewer.erase(it);
      break;
    }
  }
  glutDestroyWindow(_winId);
  _winId = -1;
}

// redisplay window
void Viewer::redisplay() {
  glutSetWindow(_winId);
  glutPostRedisplay();
}

Viewer* Viewer::getCurrentViewer() {
  int id = glutGetWindow();
  for(int i = 0; i < _viewer.size(); ++i)
    if(_viewer[i]->_winId == id)
      return _viewer[i];
  return 0;
}

// glut callbacks
void Viewer::_display() {
  glClearColor(1.0, 1.0, 1.0, 0.0);
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  Viewer *v = getCurrentViewer();
  if (v) v->display();

  glutSwapBuffers();
}

void Viewer::_reshape(int w, int h) {
  Viewer *v = getCurrentViewer();
  if (v) v->reshape(w, h);
}

void Viewer::_mouse(int button, int state, int x, int y) {
  Viewer *v = getCurrentViewer();
  if (v) v->mouse(button, state, x, y);
}

void Viewer::_motion(int x, int y) {
  Viewer *v = getCurrentViewer();
  if (v) v->motion(x, y);
}

void Viewer::_keyboard(unsigned char key, int x, int y) {
  Viewer *v = getCurrentViewer();
  if (v) v->keyboard(key, x, y);
}

void Viewer::reshape(int w, int h) {
  if ((w == 0) || (h == 0) || ((w == _width) && (h == _height))) return;
  _width = w;
  _height = h;
  glViewport(0, 0, w, h);
  glutPostRedisplay();
};
